﻿CREATE OR REPLACE FUNCTION report_row_update() RETURNS TRIGGER
LANGUAGE plpgsql
AS $$
DECLARE
	old_total INTEGER;
BEGIN
	SELECT 
		total INTO old_total
	FROM
		report_mv
	WHERE
		page=OLD.page AND
		view_date=OLD.view_time::DATE AND
		method_index=OLD.method_index;
	IF old_total=1 THEN
		DELETE FROM 
			report_mv
		WHERE
			page=OLD.page AND
			view_date=OLD.view_time::DATE AND
			method_index=OLD.method_index;
	ELSE
		UPDATE
			report_mv
		SET
			load_time=((load_time*total)-OLD.load_time)/(total-1),
			total=total-1
		WHERE
			page=OLD.page AND
			view_date=OLD.view_time::DATE AND
			method_index=OLD.method_index;
	END IF;
	SELECT 
		total INTO old_total
	FROM
		report_mv
	WHERE
		page=NEW.page AND
		view_date=NEW.view_time::DATE AND
		method_index=NEW.method_index;

	IF old_total IS NULL THEN
		INSERT INTO 
			report_mv
		VALUES
			(
			NEW.view_time::DATE,
			NEW.page,			
			NEW.method_index,
			1,
			NEW.load_time
			);
	ELSE
		UPDATE
			report_mv
		SET
			load_time=((load_time*total)+NEW.load_time)/(total+1),
			total=total+1
		WHERE
			page=NEW.page AND
			view_date=NEW.view_time::DATE AND
			method_index=NEW.method_index;
	END IF;
	RETURN NULL;	
END;
$$;


CREATE TRIGGER page_views_update AFTER UPDATE
   ON page_views FOR EACH ROW
   EXECUTE PROCEDURE public.report_row_update();
